<?php

namespace Drupal\mask_user_data\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class Mask. Contains the admin form.
 *
 * @package Drupal\mask_user_data\Form
 */
class Mask extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'mask_user_data.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mask_user_data_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('mask_user_data.settings');

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#description' => $this->t('<b>Only enable this if you are on a 
        non-production site</b>. The process is not reversible and you might lose 
        sensitive data.<br />
        For production, it is strongly recommended to load this value from the 
        <em>settings.php</em> file like: 
        <em>$conf[\'mask_user_data_enabled\'] = FALSE;</em>
        <br /><br />
        Data masking process can be triggered via drush: <b>drush mud</b><br />
        Or else it will run via <b>cron</b> hook.
      '),
      '#default_value' => $config->get('enabled') ?: FALSE,
    ];

    $map_array = $config->get('map_array') ?: NULL;
    $map_json = $map_array ? json_encode($map_array) : '';
    $form['map'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Map'),
      '#description' => $this->t('Provide a valid JSON string where the keys are
        the field names and the values are the <a target="_blank" href="https://github.com/fzaninotto/Faker#formatters">Faker functions</a>. <br/>
        Example:
        <em>{ "mail": "email", "field_job_title": "jobTitle" }</em>.<br /><br />
        * If the string is not valid JSON then no data masking will be performed,
        or if it is valid but contains invalid field names, properties or 
        functions then those invalid ones will not be masked.
      '),
      '#default_value' => $map_json,
      '#states' => [
        'invisible' => [
          ':input[name="enabled"]' => ['checked' => FALSE],
        ],
      ],
    ];

    /*
    TODO: dynamic form ready from user properties and fields that will offer
    a dropdown per field with the Faker functions available to use.
    It will generate a JSON mapping replacing the previous textarea.
     */

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $config = $this->config('mask_user_data.settings');
    $config->set('enabled', (bool)$form_state->getValue('enabled'));

    if (!empty($form_state->getValue('map'))) {
      $prev_map_array = $config->get('map_array') ?: NULL;
      $prev_map_json = $prev_map_array ? json_encode($prev_map_array) : '';

      $map = $form_state->getValue('map');
      $map_array = json_decode($map, TRUE);
      $config->set('map_array', $map_array)->save();

      if ($prev_map_json !== json_encode($map_array)) {
        $config->set('already_masked', FALSE)->save();
      }
    }
    else {
      $config->set('map_array', NULL)->save();
    }
  }

}
